%Script to create random pulse trains.  Run this and save the variable et
% as a .mat file to use in other programs.  
%
% The format of these random pulses is a stable Gaussian pulse with a
% varying component added to it. The varying component is generated by
% applying a random spectral phase to the square root of the spectrum of
% the stable Gaussian, and then filtering the result with a Gaussian in
% time.  The varying component is multiplied by a constant such that it 
% has similar intensity to the stable component (although the proper 
% constant generally should be found by trial and error if the parameters
% change).
%
% Author: Michelle Rhodes
% Georgia Institute of Technology
% email: mrhodes3@gatech.edu
% August 2012; Last revision: 5/3/2013

clear all
N           = 4096;     % size of the array
num_pulses  = 5000;     % number of pulses to average over

%Constant by which the unstable field is multiplied.  Look at the average
% intensity profile of the train and/or the distribution of the unstable
% energy to the stable energy to adjust this to what you want.
energy_coeff = 14;     

%Width of Gaussian time window.  This parameter controls how long the
% random component is in time.  The average RMS width of the random 
% components will be approximately equal to this.
time_width = 100;                

%
% Create deterministic Gaussian component of pulse, want 20 dt FWHM
%
pw = sqrt(N/pi/4.5);
x = -N/2:N/2-1;
etg = exp(-x.^2 / pw^2);
ewg = 1/sqrt(N)*fftc(etg);
spectrumg = abs(ewg).^2;

% Envelopes for the random pulses
time_envelope = exp(-x.^2/time_width^2);
freq_envelope = sqrt(spectrumg);

% Initialize the field array and a bunch of diagnostic statistics
et = zeros(num_pulses, N);
tbp_rms = zeros(1, num_pulses);
tbp_r = zeros(1, num_pulses);
sig_t_rms = zeros(1, num_pulses);
area = zeros(1,num_pulses);
sig_t_fwhm = zeros(1, num_pulses);
sig_t_fwhm_r = zeros(1, num_pulses);
sig_t_rms_r = zeros(1, num_pulses);
area_random = zeros(1, num_pulses);

if num_pulses > 0
    for k = 1:num_pulses
        % Generate the random component
        ewr = freq_envelope.*exp(1i*rand(1,N)*2*pi); %same spectrum, random phase
        etr = sqrt(N)*ifftc(ewr);
        etr = energy_coeff*etr.*(time_envelope); %apply a the time envelope and adjust energy
        
        % Save some diagnostic information about the random component only
        sig_t_fwhm_r(k) = fwhm(abs(etr).^2);
        [tbp_r(k), sig_t_rms_r(k), rms_spectral_width] = calc_tbp(etr, fftc(etr));
        area_random(k) = trapz(abs(etr).^2);
        
        % Add the components together to make the final pulse
        et(k, :) = etg + etr;
        % Save more diagnostic information about total pulse
        ew = 1/sqrt(N)*fftc(et(k,:));
        area(k) = trapz(abs( et(k,:) ).^2);
        [tbp_rms(k), sig_t_rms(k), sig_w] = calc_tbp(et(k,:), ew);
        sig_t_fwhm(k) = fwhm(abs(et(k,:)).^2);
        
    end
end

fprintf('Mean time-bandwidth product:\t%d\n', mean(tbp_rms))
fprintf('Mean unstable FWHM width:\t\t%d\n', mean(sig_t_fwhm_r))
fprintf('Mean FWHM pulse width:\t\t\t%d\n\r', mean(sig_t_fwhm))

figure('Name', 'Properties of Unstable Component Only');
fig = subplot(2,2,1);
hist(area_random./area, 50);
title('ratio of unstable to stable energy');
fig = subplot(2,2,2);
hist(tbp_r, 50);
title('RMS time-bandwidth product');
fig = subplot(2,2,3);
hist(sig_t_rms_r*2*N, 50);
title('RMS length of unstable')
xlabel('Time (\deltat)');
fig = subplot(2,2,4);
hist(sig_t_fwhm_r, 50);
title('FWHM length of unstable');
xlabel('Time (\deltat)');

figure('Name', 'Properties of Pulses');
fig = subplot(2,2,1);
hist(area, 50);
title('Total pulse energy');
fig = subplot(2,2,2);
hist(tbp_rms, 50);
title('RMS time-bandwidth product');
fig = subplot(2,2,3);
hist(sig_t_rms*2*N, 50);
title('RMS length')
xlabel('Time (\deltat)');
fig = subplot(2,2,4);
hist(sig_t_fwhm,50);
title('FWHM length');
xlabel('Time (\deltat)');


intensity = zeros(N,1);
for k = 1:N
    intensity(k) = sum(abs(et(:,k)).^2);
end
figure;
intensity = quickscale(intensity);
plot(x, intensity, 'LineWidth', 2);
axis([-300 300 0 1]);
title('Average Temporal Intensity Profie');
xlabel('Time (\deltat)');