clear all
% Combines the functionality of multipulse_spider_from_file.m and
% make_pulse_train.m into a single script that generates a pulse train and
% a SPIDER measurement of the pulse train.  Does not generate any of the 
% pulse train statistics found in make_pulse_train.m .
%
% Authors: Justin Ratner and Michelle Rhodes
% Georgia Institute of Technology
% email: mrhodes3@gatech.edu
% Last revision: 5/3/2013

N           = 4096;     % size of the array
num_pulses  = 5000;     % number of pulses to average over
freq_shift  = 9;        % SPIDER frequency shear, units 2*pi/(N*(temporal resolution))
delay       = 450;    

%Constant by which the unstable field is multiplied.
energy_coeff = 14;     

%Width of Gaussian time window.  This parameter controls how long the
% random component is in time.  The average RMS width of the random 
% components will be approximately equal to this.
time_width = 100;   

%
% Create deterministic Gaussian component of pulse, want 20 dt FWHM
%
pw = sqrt(N/pi/4.5);
x = -N/2:N/2-1;
etg = exp(-x.^2 / pw^2);
ewg = 1/sqrt(N)*fftc(etg);
spectrumg = abs(ewg).^2;

% Envelopes for the random pulses
time_envelope = exp(-x.^2/time_width^2);
freq_envelope = sqrt(spectrumg);

%add up the SPIDER traces for each pulse
et = zeros(num_pulses, N);
s = zeros (N,1);

if num_pulses > 0
    for k = 1:num_pulses
        % Generate the random component
        ewr = freq_envelope.*exp(1i*rand(1,N)*2*pi); %same spectrum, random phase
        etr = sqrt(N)*ifftc(ewr);
        etr = energy_coeff*etr.*(time_envelope); %apply a the time envelope and adjust energy
        et(k, :) = etg + etr;
        
        s_curr = spider_trace(et(k, :), freq_shift, delay);
        s = s + s_curr;
    end
end
s = quickscale(s);  %normalize the peak to 1

%
% Create a SPIDER trace for a Gaussian pulse where the bandwidth and 
% relative intensity in the arms of the SPIDER setup are parameters
% in a fit to the averaged multipulse SPIDER trace
%
pw = sqrt(N/(12*pi));  %initial guess for pulse width in fit
consts = [freq_shift delay N];
[Estimates, error] = fminsearch(@(params) spider_fit(params, consts, s), [1 pw]);
x = -N/2:N/2-1;
etg = exp(-x.^2 / (Estimates(2))^2);
sg = spider_trace_unequal_arms(etg, freq_shift, delay, Estimates(1));

%
% Plot the multipluse SPIDER trace with the fitted Gaussian SPIDER trace
%
w = x;
figure;
h = plot(w, s, '-g', w, sg, '--k');
style_plot(gca, h, 'spider')
set(h(2), 'LineWidth', 2)

%Find the peak of the background in the trace.  First, find the bottom of 
% the fringes by finding the minima, then choose the largest from them.
trace_prime = diff(s);
%assume that there should be fewer than N/2 local minima in N points
minima = zeros(N/2,1);
a = 1;
for k = 1:N-2
    if trace_prime(k) < 0 && trace_prime(k+1)>0
        minima(a) = s(k+1);
        a = a+1;
    end
end
peak = max(minima);


fprintf('Fitted pulse width:\t\t%d \n', Estimates(2))
rel_int = min([Estimates(1) 1/Estimates(1)]);
fprintf('Fitted rel. intensity:\t%d \n', rel_int)
% RMS error
rmse = sqrt(mean((s - sg).^2));
fprintf('Fitted RMSE:\t\t\t%d \n\n', rmse)
fprintf('Peak of the background:\t%d%%\n\n', peak*100);